#include "StdAfx.h"

#include "VideoManager.h"

CRITICAL_SECTION videoCS;
/**********************************************************************/
void StartCS() { EnterCriticalSection(&videoCS); }
void StopCS()  { LeaveCriticalSection(&videoCS); }
/**********************************************************************/
CVideoManager::CVideoManager(void)
{
	_colorImageData = NULL;
	_grayImageData  = NULL;
	_colorHeader    = NULL;
	_grayHeader     = NULL;

	_isVideoStreamOpen   = false;
	_isVideoStreamRecord = false;

	_maxVChannelNumber  = 0;
	_videoChannelNumber = 0;
	_compressorID		= 0;

	_videoDeviceName = _T("No video device available");

	_colorImageSize.width        = 0; 
	_colorImageSize.height       = 0; 
	_colorImageSize.colorChannel = 0;

	_grayImageSize.width         = 0; 
	_grayImageSize.height        = 0;
	_grayImageSize.colorChannel  = 0;

	_lastColorImageSize.width        = 0; 
	_lastColorImageSize.height       = 0; 
	_lastColorImageSize.colorChannel = 0;

	for(int i = 0; i < MAX_DEVICE_NUM; i++)
		_videoDeviceList[i].Empty(); 

	PrepareVideoDeviceProperty();

	InitializeCriticalSection(&videoCS);
}
CVideoManager::~CVideoManager(void)
{
	for(int i = 0; i < MAX_DEVICE_NUM; i++)
		_videoDeviceList[i].Empty(); 

	if(_videoProrertyPtr.GetInterfacePtr())
		_videoProrertyPtr.Release(); 

	if(_videoModulePtr.GetInterfacePtr())
		_videoModulePtr.Release();

	DeleteCriticalSection(&videoCS);
}
/**********************************************************************/
void CVideoManager::PrepareVideoDeviceProperty()
{
	DeviceList deviceList;

	try
	{
		if(!_videoProrertyPtr.GetInterfacePtr()) 
			_videoProrertyPtr = IVideoDevicePropertyPtr(__uuidof(VideoDeviceProperty));
	}
	catch(_com_error err)
		{
			AfxMessageBox(err.ErrorMessage()); 
			return;
		}

	deviceList = _videoProrertyPtr->GetVideoDeviceList();
	_maxVChannelNumber = deviceList.lenght;

	for(int i = 0; i < _maxVChannelNumber; i++)
	{
		int start = 0;
		int position = -1;

		_videoDeviceList[i].Format("%S", deviceList.deviceArray[i].deviceName); 

		position = _videoDeviceList[i].FindOneOf(":\?/*""<>|");
		
		if(position != -1)
			_videoDeviceList[i].SetAt(position, '-');
	}

	VideoCompressorList comprList = _videoProrertyPtr->GetVideoCompressorList();

	for(int i = 0; i < comprList.lenght; i++)
		_compressorList[i].Format("%S", comprList.compressorList[i]);
}
void CVideoManager::PrepareVideoDeviceName()
{
	int start = 0;
	int position = -1;

	_videoDeviceName.Empty(); 

	if(!_videoModulePtr.GetInterfacePtr())
		return;

	BSTR name = _videoModulePtr->GetVideoDeviceName();

	_videoDeviceName.Format("%S", name); 

	position = _videoDeviceName.FindOneOf(":\?/*""<>|");

	if(position != -1)
		_videoDeviceName.SetAt(position, '-');
}
/**********************************************************************/
bool CVideoManager::InitVideoStream(SourceDlg dialog, int videoChannel)
{
	VideoStartData startData;
	bool startResult = false;
	BOOL openStatus = FALSE;

	_isVideoStreamOpen = false;

	if(videoChannel >= _maxVChannelNumber)
		videoChannel = 0;

	if(_videoModulePtr.GetInterfacePtr())
		_videoModulePtr.Release();

	try
	{
		_videoModulePtr = IVideoModulePtr(__uuidof(VideoModule));
	}
	catch(_com_error err)
		{
			AfxMessageBox(err.ErrorMessage()); 
			return false;
		}
      
	startData.dialog = dialog;

	if(videoChannel <= -1)
		startData.videoChannel = 0;
	else
		startData.videoChannel = videoChannel;

	_videoModulePtr->OpenVideoStream(startData);    
	openStatus = _videoModulePtr->IsVideoStreamOpen(); 

	if(!openStatus)
		return false;

	_isVideoStreamOpen  = true;
	_videoChannelNumber = startData.videoChannel;

	_colorHeader = (BITMAPINFO*)_videoModulePtr->GetColorHeader();
	_grayHeader  = (BITMAPINFO*)_videoModulePtr->GetGrayHeader();

	_colorImageSize = _videoModulePtr->GetColorVideoImageSize();
	_grayImageSize  = _videoModulePtr->GetGrayVideoImageSize();

	_colorImageData = _videoModulePtr->GetColorVideoData();
	_grayImageData  = _videoModulePtr->GetGrayVideoData();

	PrepareVideoDeviceName();

	return _isVideoStreamOpen;
}
bool CVideoManager::StopVideoStream()
{
	if(!_videoModulePtr.GetInterfacePtr())
		return false;

	_videoModulePtr->StopVideoStream();
	_videoModulePtr.Release(); 

	return true;
}
bool CVideoManager::StartVideoRecord()
{
	CString fullPathName, saveFileName;
	CTime currTime = CTime::GetCurrentTime();

	StartCS();

	StopVideoRecord();

	if(_videoSaveModulePtr.GetInterfacePtr())
		_videoSaveModulePtr.Release();

	try
	{
		_videoSaveModulePtr = IVideoSaveModulePtr(__uuidof(VideoSaveModule));
	}
	catch(_com_error err)
		{
			AfxMessageBox(err.ErrorMessage()); 
			StopCS();
			return false;
		}

	char currDir[500];
	GetCurrentDirectory(500, currDir);

	saveFileName.Format("(%s)%d_%d_%d-%d_%d_%d.avi", _videoDeviceName, currTime.GetYear(), currTime.GetMonth(), currTime.GetMinute(),
													 currTime.GetHour(), currTime.GetMinute(), currTime.GetSecond());
	fullPathName = CString(currDir) + _T("\\") + saveFileName;

	_videoSaveModulePtr->SetVideoSaveFrameRate(24);
	_videoSaveModulePtr->InitVideoSaveModule(_colorHeader, _bstr_t(fullPathName), _compressorID);

	_isVideoStreamRecord = (_videoSaveModulePtr->IsVideoSaveStreamOpen() == TRUE);

	StopCS();

	return _isVideoStreamRecord;
}
bool CVideoManager::StopVideoRecord()
{
	bool result = false;

	StartCS();
	
	if(!_videoSaveModulePtr.GetInterfacePtr() || !_isVideoStreamRecord)
	{
		StopCS();
		return false;
	}

	_videoSaveModulePtr->StopVideoSaveFile();
	_videoSaveModulePtr.Release(); 

	_isVideoStreamRecord = false;

	StopCS();

	return result;
}
/**********************************************************************/
int CVideoManager::NextFrame()
{
	int result = -1;

	if(!_videoModulePtr.GetInterfacePtr())
		return result;

	_videoModulePtr->NextFrame();
	result = _videoModulePtr->GetVideoStatus();

	if(result != 1)
		return result;

	_colorImageSize = _videoModulePtr->GetColorVideoImageSize();
	_grayImageSize  = _videoModulePtr->GetGrayVideoImageSize();

	_colorImageData = _videoModulePtr->GetColorVideoData();
	_grayImageData  = _videoModulePtr->GetGrayVideoData();

	return result;
}
void CVideoManager::VideoSaveNextFrame()
{
	StartCS();

	if(!_videoSaveModulePtr.GetInterfacePtr() || !_isVideoStreamRecord || !_colorImageData)
	{
		StopCS();
		return;
	}

	_videoSaveModulePtr->PushNextFrame(_colorImageData);

	StopCS();
}
/**********************************************************************/
CString CVideoManager::GetVideoDeviceName(int index)
{
	if(index < 0 || index > _maxVChannelNumber)
		return "";

	return _videoDeviceList[index];
}